# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from saharaclient.api import base


class NodeGroupTemplate(base.Resource):
    resource_name = 'Node Group Template'


class NodeGroupTemplateManagerV1(base.ResourceManager):
    resource_class = NodeGroupTemplate
    NotUpdated = base.NotUpdated()

    def create(self, name, plugin_name, hadoop_version, flavor_id,
               description=None, volumes_per_node=None, volumes_size=None,
               node_processes=None, node_configs=None, floating_ip_pool=None,
               security_groups=None, auto_security_group=None,
               availability_zone=None, volumes_availability_zone=None,
               volume_type=None, image_id=None, is_proxy_gateway=None,
               volume_local_to_instance=None, use_autoconfig=None,
               shares=None, is_public=None, is_protected=None,
               volume_mount_prefix=None):
        """Create a Node Group Template."""

        data = {
            'name': name,
            'plugin_name': plugin_name,
            'hadoop_version': hadoop_version,
            'flavor_id': flavor_id,
            'node_processes': node_processes
        }

        return self._do_create(data, description, volumes_per_node,
                               volumes_size, node_configs, floating_ip_pool,
                               security_groups, auto_security_group,
                               availability_zone, volumes_availability_zone,
                               volume_type, image_id, is_proxy_gateway,
                               volume_local_to_instance, use_autoconfig,
                               shares, is_public, is_protected,
                               volume_mount_prefix)

    def _do_create(self, data, description, volumes_per_node, volumes_size,
                   node_configs, floating_ip_pool, security_groups,
                   auto_security_group, availability_zone,
                   volumes_availability_zone, volume_type, image_id,
                   is_proxy_gateway, volume_local_to_instance, use_autoconfig,
                   shares, is_public, is_protected, volume_mount_prefix,
                   boot_from_volume=None, boot_volume_type=None,
                   boot_volume_az=None, boot_volume_local=None):

        self._copy_if_defined(data,
                              description=description,
                              node_configs=node_configs,
                              floating_ip_pool=floating_ip_pool,
                              security_groups=security_groups,
                              auto_security_group=auto_security_group,
                              availability_zone=availability_zone,
                              image_id=image_id,
                              is_proxy_gateway=is_proxy_gateway,
                              use_autoconfig=use_autoconfig,
                              shares=shares,
                              is_public=is_public,
                              is_protected=is_protected,
                              boot_from_volume=boot_from_volume,
                              boot_volume_type=boot_volume_type,
                              boot_volume_availability_zone=boot_volume_az,
                              boot_volume_local_to_instance=boot_volume_local
                              )

        if volumes_per_node:
            data.update({"volumes_per_node": volumes_per_node,
                         "volumes_size": volumes_size})
            if volumes_availability_zone:
                data.update({"volumes_availability_zone":
                             volumes_availability_zone})
            if volume_type:
                data.update({"volume_type": volume_type})
            if volume_local_to_instance:
                data.update(
                    {"volume_local_to_instance": volume_local_to_instance})
            if volume_mount_prefix:
                data.update({"volume_mount_prefix": volume_mount_prefix})

        return self._create('/node-group-templates', data,
                            'node_group_template')

    def update(self, ng_template_id, name=NotUpdated, plugin_name=NotUpdated,
               hadoop_version=NotUpdated, flavor_id=NotUpdated,
               description=NotUpdated, volumes_per_node=NotUpdated,
               volumes_size=NotUpdated, node_processes=NotUpdated,
               node_configs=NotUpdated, floating_ip_pool=NotUpdated,
               security_groups=NotUpdated, auto_security_group=NotUpdated,
               availability_zone=NotUpdated,
               volumes_availability_zone=NotUpdated, volume_type=NotUpdated,
               image_id=NotUpdated, is_proxy_gateway=NotUpdated,
               volume_local_to_instance=NotUpdated, use_autoconfig=NotUpdated,
               shares=NotUpdated, is_public=NotUpdated,
               is_protected=NotUpdated, volume_mount_prefix=NotUpdated):
        """Update a Node Group Template."""

        data = {}
        self._copy_if_updated(
            data, name=name, plugin_name=plugin_name,
            hadoop_version=hadoop_version, flavor_id=flavor_id,
            description=description, volumes_per_node=volumes_per_node,
            volumes_size=volumes_size, node_processes=node_processes,
            node_configs=node_configs, floating_ip_pool=floating_ip_pool,
            security_groups=security_groups,
            auto_security_group=auto_security_group,
            availability_zone=availability_zone,
            volumes_availability_zone=volumes_availability_zone,
            volume_type=volume_type, image_id=image_id,
            is_proxy_gateway=is_proxy_gateway,
            volume_local_to_instance=volume_local_to_instance,
            use_autoconfig=use_autoconfig, shares=shares,
            is_public=is_public, is_protected=is_protected,
            volume_mount_prefix=volume_mount_prefix
        )

        return self._update('/node-group-templates/%s' % ng_template_id, data,
                            'node_group_template')

    def list(self, search_opts=None, marker=None,
             limit=None, sort_by=None, reverse=None):
        """Get a list of Node Group Templates."""
        query = base.get_query_string(search_opts, limit=limit, marker=marker,
                                      sort_by=sort_by, reverse=reverse)
        url = "/node-group-templates%s" % query
        return self._page(url, 'node_group_templates', limit)

    def get(self, ng_template_id):
        """Get information about a Node Group Template."""
        return self._get('/node-group-templates/%s' % ng_template_id,
                         'node_group_template')

    def delete(self, ng_template_id):
        """Delete a Node Group Template."""
        self._delete('/node-group-templates/%s' % ng_template_id)

    def export(self, ng_template_id):
        """Export a Node Group Template."""
        return self._get('/node-group-templates/%s/export' % ng_template_id)


class NodeGroupTemplateManagerV2(NodeGroupTemplateManagerV1):
    NotUpdated = base.NotUpdated()

    def create(self, name, plugin_name, plugin_version, flavor_id,
               description=None, volumes_per_node=None, volumes_size=None,
               node_processes=None, node_configs=None, floating_ip_pool=None,
               security_groups=None, auto_security_group=None,
               availability_zone=None, volumes_availability_zone=None,
               volume_type=None, image_id=None, is_proxy_gateway=None,
               volume_local_to_instance=None, use_autoconfig=None,
               shares=None, is_public=None, is_protected=None,
               volume_mount_prefix=None, boot_from_volume=None,
               boot_volume_type=None, boot_volume_availability_zone=None,
               boot_volume_local_to_instance=None):
        """Create a Node Group Template."""

        data = {
            'name': name,
            'plugin_name': plugin_name,
            'plugin_version': plugin_version,
            'flavor_id': flavor_id,
            'node_processes': node_processes
        }

        return self._do_create(data, description, volumes_per_node,
                               volumes_size, node_configs, floating_ip_pool,
                               security_groups, auto_security_group,
                               availability_zone, volumes_availability_zone,
                               volume_type, image_id, is_proxy_gateway,
                               volume_local_to_instance, use_autoconfig,
                               shares, is_public, is_protected,
                               volume_mount_prefix, boot_from_volume,
                               boot_volume_type,
                               boot_volume_availability_zone,
                               boot_volume_local_to_instance)

    def update(self, ng_template_id, name=NotUpdated, plugin_name=NotUpdated,
               plugin_version=NotUpdated, flavor_id=NotUpdated,
               description=NotUpdated, volumes_per_node=NotUpdated,
               volumes_size=NotUpdated, node_processes=NotUpdated,
               node_configs=NotUpdated, floating_ip_pool=NotUpdated,
               security_groups=NotUpdated, auto_security_group=NotUpdated,
               availability_zone=NotUpdated,
               volumes_availability_zone=NotUpdated, volume_type=NotUpdated,
               image_id=NotUpdated, is_proxy_gateway=NotUpdated,
               volume_local_to_instance=NotUpdated, use_autoconfig=NotUpdated,
               shares=NotUpdated, is_public=NotUpdated,
               is_protected=NotUpdated, volume_mount_prefix=NotUpdated,
               boot_from_volume=NotUpdated,
               boot_volume_type=NotUpdated,
               boot_volume_availability_zone=NotUpdated,
               boot_volume_local_to_instance=NotUpdated):
        """Update a Node Group Template."""

        data = {}
        self._copy_if_updated(
            data, name=name, plugin_name=plugin_name,
            plugin_version=plugin_version, flavor_id=flavor_id,
            description=description, volumes_per_node=volumes_per_node,
            volumes_size=volumes_size, node_processes=node_processes,
            node_configs=node_configs, floating_ip_pool=floating_ip_pool,
            security_groups=security_groups,
            auto_security_group=auto_security_group,
            availability_zone=availability_zone,
            volumes_availability_zone=volumes_availability_zone,
            volume_type=volume_type, image_id=image_id,
            is_proxy_gateway=is_proxy_gateway,
            volume_local_to_instance=volume_local_to_instance,
            use_autoconfig=use_autoconfig, shares=shares,
            is_public=is_public, is_protected=is_protected,
            volume_mount_prefix=volume_mount_prefix,
            boot_from_volume=boot_from_volume,
            boot_volume_type=boot_volume_type,
            boot_volume_availability_zone=boot_volume_availability_zone,
            boot_volume_local_to_instance=boot_volume_local_to_instance
        )

        return self._patch('/node-group-templates/%s' % ng_template_id, data,
                           'node_group_template')


# NOTE(jfreud): keep this around for backwards compatibility
NodeGroupTemplateManager = NodeGroupTemplateManagerV1



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from saharaclient.osc import plugin
from saharaclient.tests.unit import base


class TestDataProcessingPlugin(base.BaseTestCase):

    @mock.patch("saharaclient.api.client.Client")
    def test_make_client(self, p_client):

        instance = mock.Mock()
        instance._api_version = {"data_processing": '1.1'}
        instance.session = 'session'
        instance._region_name = 'region_name'
        instance._cli_options.data_processing_url = 'url'
        instance._interface = 'public'

        plugin.make_client(instance)
        p_client.assert_called_with(session='session',
                                    region_name='region_name',
                                    sahara_url='url',
                                    endpoint_type='public')

    @mock.patch("saharaclient.api.client.ClientV2")
    def test_make_client_v2(self, p_client):

        instance = mock.Mock()
        instance._api_version = {"data_processing": '2'}
        instance.session = 'session'
        instance._region_name = 'region_name'
        instance._cli_options.data_processing_url = 'url'
        instance._interface = 'public'

        plugin.make_client(instance)
        p_client.assert_called_with(session='session',
                                    region_name='region_name',
                                    sahara_url='url',
                                    endpoint_type='public')



# Copyright (c) 2018 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from saharaclient.api import base


class JobTemplate(base.Resource):
    resource_name = 'Job Template'


class JobTemplatesManagerV2(base.ResourceManager):
    resource_class = JobTemplate
    NotUpdated = base.NotUpdated()

    def create(self, name, type, mains=None, libs=None, description=None,
               interface=None, is_public=None, is_protected=None):
        """Create a Job Template."""
        data = {
            'name': name,
            'type': type
        }

        self._copy_if_defined(data, description=description, mains=mains,
                              libs=libs, interface=interface,
                              is_public=is_public, is_protected=is_protected)

        return self._create('/%s' % 'job-templates', data, 'job_template')

    def list(self, search_opts=None, limit=None,
             marker=None, sort_by=None, reverse=None):
        """Get a list of Job Templates."""
        query = base.get_query_string(search_opts, limit=limit, marker=marker,
                                      sort_by=sort_by, reverse=reverse)

        url = "/%s%s" % ('job-templates', query)
        return self._page(url, 'job_templates', limit)

    def get(self, job_id):
        """Get information about a Job Template."""
        return self._get('/%s/%s' % ('job-templates', job_id), 'job_template')

    def get_configs(self, job_type):
        """Get config hints for a specified Job Template type."""
        return self._get('/%s/config-hints/%s' % ('job-templates', job_type))

    def delete(self, job_id):
        """Delete a Job Template."""
        self._delete('/%s/%s' % ('job-templates', job_id))

    def update(self, job_id, name=NotUpdated, description=NotUpdated,
               is_public=NotUpdated, is_protected=NotUpdated):
        """Update a Job Template."""

        data = {}
        self._copy_if_updated(data, name=name, description=description,
                              is_public=is_public, is_protected=is_protected)

        return self._patch('/%s/%s' % ('job-templates', job_id), data)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import time

from osc_lib import exceptions
from osc_lib import utils as osc_utils
from oslo_serialization import jsonutils as json
from oslo_utils import timeutils
from oslo_utils import uuidutils

from saharaclient.api import base


def get_resource(manager, name_or_id, **kwargs):
    if uuidutils.is_uuid_like(name_or_id):
        return manager.get(name_or_id, **kwargs)
    else:
        resource = manager.find_unique(name=name_or_id)
        if kwargs:
            # we really need additional call to apply kwargs
            resource = manager.get(resource.id, **kwargs)
        return resource


def created_at_sorted(objs, reverse=False):
    return sorted(objs, key=created_at_key, reverse=reverse)


def random_name(prefix=None):
    return "%s-%s" % (prefix, uuidutils.generate_uuid()[:8])


def created_at_key(obj):
    return timeutils.parse_isotime(obj["created_at"])


def get_resource_id(manager, name_or_id):
    if uuidutils.is_uuid_like(name_or_id):
        return name_or_id
    else:
        return manager.find_unique(name=name_or_id).id


def create_dict_from_kwargs(**kwargs):
    return {k: v for (k, v) in kwargs.items() if v is not None}


def prepare_data(data, fields):
    new_data = {}
    for f in fields:
        if f in data:
            new_data[f.replace('_', ' ').capitalize()] = data[f]

    return new_data


def unzip(data):
    return zip(*data)


def extend_columns(columns, items):
    return unzip(list(unzip(columns)) + [('', '')] + items)


def prepare_column_headers(columns, remap=None):
    remap = remap if remap else {}
    new_columns = []
    for c in columns:
        for old, new in remap.items():
            c = c.replace(old, new)
        new_columns.append(c.replace('_', ' ').capitalize())

    return new_columns


def get_by_name_substring(data, name):
    return [obj for obj in data if name in obj.name]


def wait_for_delete(manager, obj_id, sleep_time=5, timeout=3000):
    s_time = timeutils.utcnow()
    while timeutils.delta_seconds(s_time, timeutils.utcnow()) < timeout:
        try:
            manager.get(obj_id)
        except base.APIException as ex:
            if ex.error_code == 404:
                return True
            raise
        time.sleep(sleep_time)

    return False


def get_api_version(app):
    return app.api_version['data_processing']


def is_api_v2(app):
    if get_api_version(app) == '2':
        return True
    return False


def _cluster_templates_configure_ng(app, node_groups, client):
    node_groups_list = dict(
        map(lambda x: x.split(':', 1), node_groups))

    node_groups = []
    plugins_versions = set()

    for name, count in node_groups_list.items():
        ng = get_resource(client.node_group_templates, name)
        node_groups.append({'name': ng.name,
                            'count': int(count),
                            'node_group_template_id': ng.id})
        if is_api_v2(app):
            plugins_versions.add((ng.plugin_name, ng.plugin_version))
        else:
            plugins_versions.add((ng.plugin_name, ng.hadoop_version))

    if len(plugins_versions) != 1:
        raise exceptions.CommandError('Node groups with the same plugins '
                                      'and versions must be specified')

    plugin, plugin_version = plugins_versions.pop()
    return plugin, plugin_version, node_groups


def _get_plugin_version(app, cluster_template, client):
    ct = get_resource(client.cluster_templates, cluster_template)
    if is_api_v2(app):
        return ct.plugin_name, ct.plugin_version, ct.id
    else:
        return ct.plugin_name, ct.hadoop_version, ct.id


def create_job_templates(app, client, mains_ids, libs_ids, parsed_args):
    args_dict = dict(name=parsed_args.name,
                     type=parsed_args.type,
                     mains=mains_ids,
                     libs=libs_ids,
                     description=parsed_args.description,
                     interface=parsed_args.interface,
                     is_public=parsed_args.public,
                     is_protected=parsed_args.protected)

    if is_api_v2(app):
        data = client.job_templates.create(**args_dict).to_dict()
    else:
        data = client.jobs.create(**args_dict).to_dict()

    return data


def create_job_template_json(app, client, **template):
    if is_api_v2(app):
        data = client.job_templates.create(**template).to_dict()
    else:
        data = client.jobs.create(**template).to_dict()

    return data


def list_job_templates(app, client, search_opts):
    if is_api_v2(app):
        data = client.job_templates.list(search_opts=search_opts)
    else:
        data = client.jobs.list(search_opts=search_opts)

    return data


def get_job_templates_resources(app, client, parsed_args):
    if is_api_v2(app):
        data = get_resource(
            client.job_templates, parsed_args.job_template).to_dict()
    else:
        data = get_resource(
            client.jobs, parsed_args.job_template).to_dict()

    return data


def delete_job_templates(app, client, jt):
    if is_api_v2(app):
        jt_id = get_resource_id(client.job_templates, jt)
        client.job_templates.delete(jt_id)
    else:
        jt_id = get_resource_id(client.jobs, jt)
        client.jobs.delete(jt_id)


def get_job_template_id(app, client, parsed_args):
    if is_api_v2(app):
        jt_id = get_resource_id(
            client.job_templates, parsed_args.job_template)
    else:
        jt_id = get_resource_id(
            client.jobs, parsed_args.job_template)

    return jt_id


def update_job_templates(app, client, jt_id, update_data):
    if is_api_v2(app):
        data = client.job_templates.update(jt_id, **update_data).job_template
    else:
        data = client.jobs.update(jt_id, **update_data).job

    return data


def create_cluster_template(app, client, plugin, plugin_version,
                            parsed_args, configs, shares, node_groups):

    args_dict = dict(
        name=parsed_args.name,
        plugin_name=plugin,
        description=parsed_args.description,
        node_groups=node_groups,
        use_autoconfig=parsed_args.autoconfig,
        cluster_configs=configs,
        shares=shares,
        is_public=parsed_args.public,
        is_protected=parsed_args.protected,
        domain_name=parsed_args.domain_name)

    if is_api_v2(app):
        args_dict['plugin_version'] = plugin_version
    else:
        args_dict['hadoop_version'] = plugin_version

    data = client.cluster_templates.create(**args_dict).to_dict()
    return data


def update_cluster_template(app, client, plugin, plugin_version,
                            parsed_args, configs, shares, node_groups, ct_id):

    args_dict = dict(
        name=parsed_args.name,
        plugin_name=plugin,
        description=parsed_args.description,
        node_groups=node_groups,
        use_autoconfig=parsed_args.use_autoconfig,
        cluster_configs=configs,
        shares=shares,
        is_public=parsed_args.is_public,
        is_protected=parsed_args.is_protected,
        domain_name=parsed_args.domain_name
    )

    if is_api_v2(app):
        args_dict['plugin_version'] = plugin_version
    else:
        args_dict['hadoop_version'] = plugin_version

    update_dict = create_dict_from_kwargs(**args_dict)
    data = client.cluster_templates.update(
        ct_id, **update_dict).to_dict()

    return data


def create_cluster(client, app, parsed_args, plugin, plugin_version,
                   template_id, image_id, net_id):

    args = dict(
        name=parsed_args.name,
        plugin_name=plugin,
        cluster_template_id=template_id,
        default_image_id=image_id,
        description=parsed_args.description,
        is_transient=parsed_args.transient,
        user_keypair_id=parsed_args.user_keypair,
        net_id=net_id,
        count=parsed_args.count,
        is_public=parsed_args.public,
        is_protected=parsed_args.protected)

    if is_api_v2(app):
        args['plugin_version'] = plugin_version
    else:
        args['hadoop_version'] = plugin_version

    data = client.clusters.create(**args).to_dict()
    return data


def create_job(client, app, jt_id, cluster_id, input_id, output_id,
               job_configs, parsed_args):
    args_dict = dict(cluster_id=cluster_id,
                     input_id=input_id,
                     output_id=output_id,
                     interface=parsed_args.interface,
                     configs=job_configs,
                     is_public=parsed_args.public,
                     is_protected=parsed_args.protected)

    if is_api_v2(app):
        args_dict['job_template_id'] = jt_id
        data = client.jobs.create(**args_dict).to_dict()
    else:
        args_dict['job_id'] = jt_id
        data = client.job_executions.create(**args_dict).to_dict()

    return data


def create_job_json(client, app, **template):
    if is_api_v2(app):
        data = client.jobs.create(**template).to_dict()
    else:
        data = client.job_executions.create(**template).to_dict()

    return data


def update_job(client, app, parsed_args, update_dict):
    if is_api_v2(app):
        data = client.jobs.update(
            parsed_args.job, **update_dict).job
    else:
        data = client.job_executions.update(
            parsed_args.job, **update_dict).job_execution
    return data


def create_node_group_templates(client, app,  parsed_args, flavor_id, configs,
                                shares):
    if app.api_version['data_processing'] == '2':
        data = client.node_group_templates.create(
            name=parsed_args.name,
            plugin_name=parsed_args.plugin,
            plugin_version=parsed_args.plugin_version,
            flavor_id=flavor_id,
            description=parsed_args.description,
            volumes_per_node=parsed_args.volumes_per_node,
            volumes_size=parsed_args.volumes_size,
            node_processes=parsed_args.processes,
            floating_ip_pool=parsed_args.floating_ip_pool,
            security_groups=parsed_args.security_groups,
            auto_security_group=parsed_args.auto_security_group,
            availability_zone=parsed_args.availability_zone,
            volume_type=parsed_args.volumes_type,
            is_proxy_gateway=parsed_args.proxy_gateway,
            volume_local_to_instance=parsed_args.volumes_locality,
            use_autoconfig=parsed_args.autoconfig,
            is_public=parsed_args.public,
            is_protected=parsed_args.protected,
            node_configs=configs,
            shares=shares,
            volumes_availability_zone=(
                parsed_args.volumes_availability_zone),
            volume_mount_prefix=parsed_args.volumes_mount_prefix,
            boot_from_volume=parsed_args.boot_from_volume,
            boot_volume_type=parsed_args.boot_volume_type,
            boot_volume_availability_zone=(
                parsed_args.boot_volume_availability_zone),
            boot_volume_local_to_instance=(
                parsed_args.boot_volume_local_to_instance)
            ).to_dict()
    else:
        data = client.node_group_templates.create(
            name=parsed_args.name,
            plugin_name=parsed_args.plugin,
            hadoop_version=parsed_args.plugin_version,
            flavor_id=flavor_id,
            description=parsed_args.description,
            volumes_per_node=parsed_args.volumes_per_node,
            volumes_size=parsed_args.volumes_size,
            node_processes=parsed_args.processes,
            floating_ip_pool=parsed_args.floating_ip_pool,
            security_groups=parsed_args.security_groups,
            auto_security_group=parsed_args.auto_security_group,
            availability_zone=parsed_args.availability_zone,
            volume_type=parsed_args.volumes_type,
            is_proxy_gateway=parsed_args.proxy_gateway,
            volume_local_to_instance=parsed_args.volumes_locality,
            use_autoconfig=parsed_args.autoconfig,
            is_public=parsed_args.public,
            is_protected=parsed_args.protected,
            node_configs=configs,
            shares=shares,
            volumes_availability_zone=(
                parsed_args.volumes_availability_zone),
            volume_mount_prefix=parsed_args.volumes_mount_prefix).to_dict()
    return data


class NodeGroupTemplatesUtils(object):

    def _create_take_action(self, client, app, parsed_args):
        if parsed_args.json:
            blob = osc_utils.read_blob_file_contents(parsed_args.json)
            try:
                template = json.loads(blob)
            except ValueError as e:
                raise exceptions.CommandError(
                    'An error occurred when reading '
                    'template from file %s: %s' % (parsed_args.json, e))
            data = client.node_group_templates.create(**template).to_dict()
        else:
            if (not parsed_args.name or not parsed_args.plugin or
                    not parsed_args.plugin_version or not parsed_args.flavor or
                    not parsed_args.processes):
                raise exceptions.CommandError(
                    'At least --name, --plugin, --plugin-version, --processes,'
                    ' --flavor arguments should be specified or json template '
                    'should be provided with --json argument')

            configs = None
            if parsed_args.configs:
                blob = osc_utils.read_blob_file_contents(parsed_args.configs)
                try:
                    configs = json.loads(blob)
                except ValueError as e:
                    raise exceptions.CommandError(
                        'An error occurred when reading '
                        'configs from file %s: %s' % (parsed_args.configs, e))

            shares = None
            if parsed_args.shares:
                blob = osc_utils.read_blob_file_contents(parsed_args.shares)
                try:
                    shares = json.loads(blob)
                except ValueError as e:
                    raise exceptions.CommandError(
                        'An error occurred when reading '
                        'shares from file %s: %s' % (parsed_args.shares, e))

            compute_client = app.client_manager.compute
            flavor_id = osc_utils.find_resource(
                compute_client.flavors, parsed_args.flavor).id

            data = create_node_group_templates(client, app, parsed_args,
                                               flavor_id, configs, shares)

        return data

    def _list_take_action(self, client, app, parsed_args):
        search_opts = {}
        if parsed_args.plugin:
            search_opts['plugin_name'] = parsed_args.plugin
        if parsed_args.plugin_version:
            search_opts['hadoop_version'] = parsed_args.plugin_version

        data = client.node_group_templates.list(search_opts=search_opts)

        if parsed_args.name:
            data = get_by_name_substring(data, parsed_args.name)

        if app.api_version['data_processing'] == '2':
            if parsed_args.long:
                columns = ('name', 'id', 'plugin_name', 'plugin_version',
                           'node_processes', 'description')
                column_headers = prepare_column_headers(columns)

            else:
                columns = ('name', 'id', 'plugin_name', 'plugin_version')
                column_headers = prepare_column_headers(columns)
        else:
            if parsed_args.long:
                columns = ('name', 'id', 'plugin_name', 'hadoop_version',
                           'node_processes', 'description')
                column_headers = prepare_column_headers(
                    columns, {'hadoop_version': 'plugin_version'})

            else:
                columns = ('name', 'id', 'plugin_name', 'hadoop_version')
                column_headers = prepare_column_headers(
                    columns, {'hadoop_version': 'plugin_version'})

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns,
                formatters={
                    'node_processes': osc_utils.format_list
                }
            ) for s in data)
        )

    def _update_take_action(self, client, app, parsed_args):
        ngt_id = get_resource_id(
            client.node_group_templates, parsed_args.node_group_template)

        if parsed_args.json:
            blob = osc_utils.read_blob_file_contents(parsed_args.json)
            try:
                template = json.loads(blob)
            except ValueError as e:
                raise exceptions.CommandError(
                    'An error occurred when reading '
                    'template from file %s: %s' % (parsed_args.json, e))
            data = client.node_group_templates.update(
                ngt_id, **template).to_dict()
        else:
            configs = None
            if parsed_args.configs:
                blob = osc_utils.read_blob_file_contents(parsed_args.configs)
                try:
                    configs = json.loads(blob)
                except ValueError as e:
                    raise exceptions.CommandError(
                        'An error occurred when reading '
                        'configs from file %s: %s' % (parsed_args.configs, e))

            shares = None
            if parsed_args.shares:
                blob = osc_utils.read_blob_file_contents(parsed_args.shares)
                try:
                    shares = json.loads(blob)
                except ValueError as e:
                    raise exceptions.CommandError(
                        'An error occurred when reading '
                        'shares from file %s: %s' % (parsed_args.shares, e))

            flavor_id = None
            if parsed_args.flavor:
                compute_client = self.app.client_manager.compute
                flavor_id = osc_utils.find_resource(
                    compute_client.flavors, parsed_args.flavor).id

            update_dict = create_dict_from_kwargs(
                name=parsed_args.name,
                plugin_name=parsed_args.plugin,
                hadoop_version=parsed_args.plugin_version,
                flavor_id=flavor_id,
                description=parsed_args.description,
                volumes_per_node=parsed_args.volumes_per_node,
                volumes_size=parsed_args.volumes_size,
                node_processes=parsed_args.processes,
                floating_ip_pool=parsed_args.floating_ip_pool,
                security_groups=parsed_args.security_groups,
                auto_security_group=parsed_args.use_auto_security_group,
                availability_zone=parsed_args.availability_zone,
                volume_type=parsed_args.volumes_type,
                is_proxy_gateway=parsed_args.is_proxy_gateway,
                volume_local_to_instance=parsed_args.volume_locality,
                use_autoconfig=parsed_args.use_autoconfig,
                is_public=parsed_args.is_public,
                is_protected=parsed_args.is_protected,
                node_configs=configs,
                shares=shares,
                volumes_availability_zone=(
                    parsed_args.volumes_availability_zone),
                volume_mount_prefix=parsed_args.volumes_mount_prefix
            )

            if app.api_version['data_processing'] == '2':
                if 'hadoop_version' in update_dict:
                    update_dict.pop('hadoop_version')
                    update_dict['plugin_version'] = parsed_args.plugin_version
                if parsed_args.boot_from_volume is not None:
                    update_dict['boot_from_volume'] = (
                        parsed_args.boot_from_volume)
                if parsed_args.boot_volume_type is not None:
                    update_dict['boot_volume_type'] = (
                        parsed_args.boot_volume_type)
                if parsed_args.boot_volume_availability_zone is not None:
                    update_dict['boot_volume_availability_zone'] = (
                        parsed_args.boot_volume_availability_zone)
                if parsed_args.boot_volume_local_to_instance is not None:
                    update_dict['boot_volume_local_to_instance'] = (
                        parsed_args.boot_volume_local_to_instance)
            data = client.node_group_templates.update(
                ngt_id, **update_dict).to_dict()

        return data

    def _import_take_action(self, client, parsed_args):
        if (not parsed_args.image_id or
                not parsed_args.flavor_id):
            raise exceptions.CommandError(
                'At least --image_id and --flavor_id should be specified')
        blob = osc_utils.read_blob_file_contents(parsed_args.json)
        try:
            template = json.loads(blob)
        except ValueError as e:
            raise exceptions.CommandError(
                'An error occurred when reading '
                'template from file %s: %s' % (parsed_args.json, e))
        template['node_group_template']['floating_ip_pool'] = (
            parsed_args.floating_ip_pool)
        template['node_group_template']['image_id'] = (
            parsed_args.image_id)
        template['node_group_template']['flavor_id'] = (
            parsed_args.flavor_id)
        template['node_group_template']['security_groups'] = (
            parsed_args.security_groups)
        if parsed_args.name:
            template['node_group_template']['name'] = parsed_args.name
        data = client.node_group_templates.create(
            **template['node_group_template']).to_dict()

        return data

    def _export_take_action(self, client, parsed_args):
        ngt_id = get_resource_id(
            client.node_group_templates, parsed_args.node_group_template)
        response = client.node_group_templates.export(ngt_id)
        result = json.dumps(response._info, indent=4)+"\n"
        if parsed_args.file:
            with open(parsed_args.file, "w+") as file:
                file.write(result)
        else:
            sys.stdout.write(result)



# Copyright (c) 2018 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from saharaclient.api import base


class Job(base.Resource):
    resource_name = 'Job'


class JobsManagerV2(base.ResourceManager):
    resource_class = Job
    NotUpdated = base.NotUpdated()

    def list(self, search_opts=None, marker=None, limit=None,
             sort_by=None, reverse=None):
        """Get a list of Jobs."""
        query = base.get_query_string(search_opts, limit=limit, marker=marker,
                                      sort_by=sort_by, reverse=reverse)
        url = "/jobs%s" % query
        return self._page(url, 'jobs', limit)

    def get(self, obj_id):
        """Get information about a Job."""
        return self._get('/jobs/%s' % obj_id, 'job')

    def delete(self, obj_id):
        """Delete a Job."""
        self._delete('/jobs/%s' % obj_id)

    def create(self, job_template_id, cluster_id, input_id=None,
               output_id=None, configs=None, interface=None, is_public=None,
               is_protected=None):
        """Launch a Job."""

        data = {
            "cluster_id": cluster_id,
            "job_template_id": job_template_id
        }

        self._copy_if_defined(data, input_id=input_id, output_id=output_id,
                              job_configs=configs, interface=interface,
                              is_public=is_public, is_protected=is_protected)

        return self._create('/jobs', data, 'job')

    def refresh_status(self, obj_id):
        """Refresh Job Status."""
        return self._get(
            '/jobs/%s?refresh_status=True' % obj_id,
            'job'
        )

    def update(self, obj_id, is_public=NotUpdated, is_protected=NotUpdated):
        """Update a Job."""

        data = {}
        self._copy_if_updated(data, is_public=is_public,
                              is_protected=is_protected)
        return self._patch('/jobs/%s' % obj_id, data)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneauth1 import adapter

from saharaclient.api import cluster_templates
from saharaclient.api import clusters
from saharaclient.api import data_sources
from saharaclient.api import images
from saharaclient.api import job_binaries
from saharaclient.api import job_binary_internals
from saharaclient.api import job_executions
from saharaclient.api import job_types
from saharaclient.api import jobs
from saharaclient.api import node_group_templates
from saharaclient.api import plugins
from saharaclient.api.v2 import job_templates
from saharaclient.api.v2 import jobs as jobs_v2


USER_AGENT = 'python-saharaclient'


class HTTPClient(adapter.Adapter):

    def request(self, *args, **kwargs):
        kwargs.setdefault('raise_exc', False)
        kwargs.setdefault('allow', {'allow_experimental': True})
        return super(HTTPClient, self).request(*args, **kwargs)


class Client(object):

    _api_version = '1.1'

    """Client for the OpenStack Data Processing API.
        :param session: Keystone session object. Required.
        :param string sahara_url: Endpoint override.
        :param string endpoint_type: Desired Sahara endpoint type.
        :param string service_type: Sahara service name in Keystone catalog.
        :param string region_name: Name of a region to select when choosing an
                                   endpoint from the service catalog.
    """
    def __init__(self, session=None, sahara_url=None,
                 endpoint_type='publicURL', service_type='data-processing',
                 region_name=None, **kwargs):

        if not session:
            raise RuntimeError("Must provide session")

        auth = session.auth

        kwargs['user_agent'] = USER_AGENT
        kwargs.setdefault('interface', endpoint_type)
        kwargs.setdefault('endpoint_override', sahara_url)

        client = HTTPClient(session=session,
                            auth=auth,
                            service_type=service_type,
                            region_name=region_name,
                            version=self._api_version,
                            **kwargs)

        self._register_managers(client)

    def _register_managers(self, client):
        self.clusters = clusters.ClusterManagerV1(client)
        self.cluster_templates = (
            cluster_templates.ClusterTemplateManagerV1(client)
        )
        self.node_group_templates = (
            node_group_templates.NodeGroupTemplateManagerV1(client)
        )
        self.plugins = plugins.PluginManagerV1(client)
        self.images = images.ImageManagerV1(client)
        self.data_sources = data_sources.DataSourceManagerV1(client)
        self.jobs = jobs.JobsManagerV1(client)
        self.job_executions = job_executions.JobExecutionsManager(client)
        self.job_binaries = job_binaries.JobBinariesManagerV1(client)
        self.job_binary_internals = (
            job_binary_internals.JobBinaryInternalsManager(client)
        )
        self.job_types = job_types.JobTypesManager(client)


class ClientV2(Client):

    _api_version = '2'

    def _register_managers(self, client):
        self.clusters = clusters.ClusterManagerV2(client)
        self.cluster_templates = (
            cluster_templates.ClusterTemplateManagerV2(client)
        )
        self.node_group_templates = (
            node_group_templates.NodeGroupTemplateManagerV2(client)
        )
        self.plugins = plugins.PluginManagerV2(client)
        self.images = images.ImageManagerV2(client)
        self.data_sources = data_sources.DataSourceManagerV2(client)
        self.job_templates = job_templates.JobTemplatesManagerV2(client)
        self.jobs = jobs_v2.JobsManagerV2(client)
        self.job_binaries = job_binaries.JobBinariesManagerV2(client)
        self.job_types = job_types.JobTypesManager(client)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from saharaclient.api import job_executions as je
from saharaclient.tests.unit import base

from oslo_serialization import jsonutils as json


class JobExecutionTest(base.BaseTestCase):
    body = {
        'job_id': 'job_id',
        'cluster_id': 'cluster_id',
        'configs': {},
        'interface': {},
        'input_id': None,
        'output_id': None
    }
    response = {
        'cluster_id': 'cluster_id',
        'interface': {},
        'job_configs': {}
    }

    update_json = {
        'is_public': True,
        'is_protected': True,
    }

    def test_create_job_execution_with_io(self):
        url = self.URL + '/jobs/job_id/execute'

        body = self.body.copy()
        body.update({'input_id': 'input_id', 'output_id': 'output_id'})
        response = self.response.copy()
        response.update({'input_id': 'input_id', 'output_id': 'output_id'})

        self.responses.post(url, status_code=202,
                            json={'job_execution': response})

        resp = self.client.job_executions.create(**body)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(response,
                         json.loads(self.responses.last_request.body))
        self.assertIsInstance(resp, je.JobExecution)
        self.assertFields(response, resp)

    def test_create_job_execution_without_io(self):
        url = self.URL + '/jobs/job_id/execute'

        self.responses.post(url, status_code=202,
                            json={'job_execution': self.response})

        resp = self.client.job_executions.create(**self.body)

        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(self.response,
                         json.loads(self.responses.last_request.body))
        self.assertIsInstance(resp, je.JobExecution)
        self.assertFields(self.response, resp)

    def test_job_executions_list(self):
        url = self.URL + '/job-executions'
        self.responses.get(url, json={'job_executions': [self.response]})

        resp = self.client.job_executions.list()

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp[0], je.JobExecution)
        self.assertFields(self.response, resp[0])

    def test_job_executions_get(self):
        url = self.URL + '/job-executions/id'
        self.responses.get(url, json={'job_execution': self.response})

        resp = self.client.job_executions.get('id')

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, je.JobExecution)
        self.assertFields(self.response, resp)

    def test_job_executions_delete(self):
        url = self.URL + '/job-executions/id'
        self.responses.delete(url, status_code=204)

        self.client.job_executions.delete('id')

        self.assertEqual(url, self.responses.last_request.url)

    def test_job_executions_update(self):
        url = self.URL + '/job-executions/id'
        self.responses.patch(url, status_code=202, json=self.update_json)

        # check that all parameters will be updated
        resp = self.client.job_executions.update("id", **self.update_json)
        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp, je.JobExecution)
        self.assertEqual(self.update_json,
                         json.loads(self.responses.last_request.body))

        # check that parameters will not be updated
        self.client.job_executions.update("id")
        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual({},
                         json.loads(self.responses.last_request.body))

        # check that all parameters will be unset
        unset_json = {
            "is_public": None, "is_protected": None
        }

        self.client.job_executions.update("id", **unset_json)
        self.assertEqual(url, self.responses.last_request.url)
        self.assertEqual(unset_json,
                         json.loads(self.responses.last_request.body))



# Copyright (c) 2015 Red Hat Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from saharaclient.api import job_types as jt
from saharaclient.tests.unit import base


class JobTypesTest(base.BaseTestCase):
    body = {
        "name": "Hive",
        "plugins": [
            {
                "description": "The Apache Vanilla plugin.",
                "name": "vanilla",
                "title": "Vanilla Apache Hadoop",
                "versions": {
                    "1.2.1": {}
                }
            },
            {
                "description": "The Hortonworks Sahara plugin.",
                "name": "hdp",
                "title": "Hortonworks Data Platform",
                "versions": {
                    "1.3.2": {},
                    "2.0.6": {}
                }
            }
        ]
    }

    def test_job_types_list(self):
        url = self.URL + '/job-types'
        self.responses.get(url, json={'job_types': [self.body]})

        resp = self.client.job_types.list()

        self.assertEqual(url, self.responses.last_request.url)
        self.assertIsInstance(resp[0], jt.JobType)
        self.assertFields(self.body, resp[0])



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from os import path
import sys

from oslo_log import log as logging
from oslo_serialization import jsonutils

from saharaclient.osc.v1 import job_types as jt_v1


class ListJobTypes(jt_v1.ListJobTypes):
    """Lists job types supported by plugins"""

    log = logging.getLogger(__name__ + ".ListJobTypes")


class GetJobTypeConfigs(jt_v1.GetJobTypeConfigs):
    """Get job type configs"""

    log = logging.getLogger(__name__ + ".GetJobTypeConfigs")

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        if not parsed_args.file:
            parsed_args.file = parsed_args.job_type

        data = client.job_templates.get_configs(parsed_args.job_type).to_dict()

        if path.exists(parsed_args.file):
            self.log.error('File "%s" already exists. Choose another one with '
                           '--file argument.' % parsed_args.file)
        else:
            with open(parsed_args.file, 'w') as f:
                jsonutils.dump(data, f, indent=4)
            sys.stdout.write(
                '"%(type)s" job configs were saved in "%(file)s"'
                'file' % {'type': parsed_args.job_type,
                          'file': parsed_args.file})



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from saharaclient.osc.v1 import data_sources as ds_v1


class CreateDataSource(ds_v1.CreateDataSource):
    """Creates data source"""

    log = logging.getLogger(__name__ + ".CreateDataSource")


class ListDataSources(ds_v1.ListDataSources):
    """Lists data sources"""

    log = logging.getLogger(__name__ + ".ListDataSources")


class ShowDataSource(ds_v1.ShowDataSource):
    """Display data source details"""

    log = logging.getLogger(__name__ + ".ShowDataSource")


class DeleteDataSource(ds_v1.DeleteDataSource):
    """Delete data source"""

    log = logging.getLogger(__name__ + ".DeleteDataSource")


class UpdateDataSource(ds_v1.UpdateDataSource):
    """Update data source"""

    log = logging.getLogger(__name__ + ".UpdateDataSource")



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils
from oslo_log import log as logging
from oslo_serialization import jsonutils as json

from saharaclient.osc import utils

CT_FIELDS = ['id', 'name', 'plugin_name', 'plugin_version', 'description',
             'node_groups', 'anti_affinity', 'use_autoconfig', 'is_default',
             'is_protected', 'is_public', 'domain_name']


def _format_node_groups_list(node_groups):
    return ', '.join(
        ['%s:%s' % (ng['name'], ng['count']) for ng in node_groups])


def _format_ct_output(app, data):
    data['plugin_version'] = data.pop('hadoop_version')
    data['node_groups'] = _format_node_groups_list(data['node_groups'])
    data['anti_affinity'] = osc_utils.format_list(data['anti_affinity'])


def _configure_node_groups(app, node_groups, client):
    node_groups_list = dict(
        map(lambda x: x.split(':', 1), node_groups))

    node_groups = []
    plugins_versions = set()

    for name, count in node_groups_list.items():
        ng = utils.get_resource(client.node_group_templates, name)
        node_groups.append({'name': ng.name,
                            'count': int(count),
                            'node_group_template_id': ng.id})
        plugins_versions.add((ng.plugin_name, ng.hadoop_version))

    if len(plugins_versions) != 1:
        raise exceptions.CommandError('Node groups with the same plugins '
                                      'and versions must be specified')

    plugin, plugin_version = plugins_versions.pop()
    return plugin, plugin_version, node_groups


class CreateClusterTemplate(command.ShowOne):
    """Creates cluster template"""

    log = logging.getLogger(__name__ + ".CreateClusterTemplate")

    def get_parser(self, prog_name):
        parser = super(CreateClusterTemplate, self).get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar="<name>",
            help="Name of the cluster template [REQUIRED if JSON is not "
                 "provided]",
        )
        parser.add_argument(
            '--node-groups',
            metavar="<node-group:instances_count>",
            nargs="+",
            help="List of the node groups(names or IDs) and numbers of "
                 "instances for each one of them [REQUIRED if JSON is not "
                 "provided]"
        )
        parser.add_argument(
            '--anti-affinity',
            metavar="<anti-affinity>",
            nargs="+",
            help="List of processes that should be added to an anti-affinity "
                 "group"
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help='Description of the cluster template'
        )
        parser.add_argument(
            '--autoconfig',
            action='store_true',
            default=False,
            help='If enabled, instances of the cluster will be '
                 'automatically configured',
        )
        parser.add_argument(
            '--public',
            action='store_true',
            default=False,
            help='Make the cluster template public (Visible from other '
                 'projects)',
        )
        parser.add_argument(
            '--protected',
            action='store_true',
            default=False,
            help='Make the cluster template protected',
        )
        parser.add_argument(
            '--json',
            metavar='<filename>',
            help='JSON representation of the cluster template. Other '
                 'arguments will not be taken into account if this one is '
                 'provided'
        )
        parser.add_argument(
            '--shares',
            metavar='<filename>',
            help='JSON representation of the manila shares'
        )
        parser.add_argument(
            '--configs',
            metavar='<filename>',
            help='JSON representation of the cluster template configs'
        )
        parser.add_argument(
            '--domain-name',
            metavar='<domain-name>',
            help='Domain name for instances of this cluster template. This '
                 'option is available if \'use_designate\' config is True'
        )
        return parser

    def _take_action(self, client, parsed_args):
        if parsed_args.json:
            blob = osc_utils.read_blob_file_contents(parsed_args.json)
            try:
                template = json.loads(blob)
            except ValueError as e:
                raise exceptions.CommandError(
                    'An error occurred when reading '
                    'template from file %s: %s' % (parsed_args.json, e))

            if 'neutron_management_network' in template:
                template['net_id'] = template.pop('neutron_management_network')

            data = client.cluster_templates.create(**template).to_dict()
        else:
            if not parsed_args.name or not parsed_args.node_groups:
                raise exceptions.CommandError(
                    'At least --name , --node-groups arguments should be '
                    'specified or json template should be provided with '
                    '--json argument')

            configs = None
            if parsed_args.configs:
                blob = osc_utils.read_blob_file_contents(parsed_args.configs)
                try:
                    configs = json.loads(blob)
                except ValueError as e:
                    raise exceptions.CommandError(
                        'An error occurred when reading '
                        'configs from file %s: %s' % (parsed_args.configs, e))

            shares = None
            if parsed_args.shares:
                blob = osc_utils.read_blob_file_contents(parsed_args.shares)
                try:
                    shares = json.loads(blob)
                except ValueError as e:
                    raise exceptions.CommandError(
                        'An error occurred when reading '
                        'shares from file %s: %s' % (parsed_args.shares, e))

            plugin, plugin_version, node_groups = (
                utils._cluster_templates_configure_ng(self.app,
                                                      parsed_args.node_groups,
                                                      client))
            data = utils.create_cluster_template(self.app, client, plugin,
                                                 plugin_version,
                                                 parsed_args, configs, shares,
                                                 node_groups)

        return data

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        _format_ct_output(self.app, data)
        data = utils.prepare_data(data, CT_FIELDS)

        return self.dict2columns(data)


class ListClusterTemplates(command.Lister):
    """Lists cluster templates"""

    log = logging.getLogger(__name__ + ".ListClusterTemplates")

    def get_parser(self, prog_name):
        parser = super(ListClusterTemplates, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help='List additional fields in output',
        )
        parser.add_argument(
            '--plugin',
            metavar="<plugin>",
            help="List cluster templates for specific plugin"
        )

        parser.add_argument(
            '--plugin-version',
            metavar="<plugin_version>",
            help="List cluster templates with specific version of the "
                 "plugin"
        )

        parser.add_argument(
            '--name',
            metavar="<name-substring>",
            help="List cluster templates with specific substring in the "
                 "name"
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        search_opts = {}
        if parsed_args.plugin:
            search_opts['plugin_name'] = parsed_args.plugin
        if parsed_args.plugin_version:
            if utils.is_api_v2(self.app):
                search_opts['plugin_version'] = parsed_args.plugin_version
            else:
                search_opts['hadoop_version'] = parsed_args.plugin_version

        data = client.cluster_templates.list(search_opts=search_opts)

        if parsed_args.name:
            data = utils.get_by_name_substring(data, parsed_args.name)

        if parsed_args.long:
            columns = ('name', 'id', 'plugin_name', 'hadoop_version',
                       'node_groups', 'description')
            column_headers = utils.prepare_column_headers(
                columns, {'hadoop_version': 'plugin_version'})

        else:
            columns = ('name', 'id', 'plugin_name', 'hadoop_version')
            column_headers = utils.prepare_column_headers(
                columns, {'hadoop_version': 'plugin_version'})

        return (
            column_headers,
            (osc_utils.get_item_properties(
                s,
                columns,
                formatters={
                    'node_groups': _format_node_groups_list
                }
            ) for s in data)
        )


class ShowClusterTemplate(command.ShowOne):
    """Display cluster template details"""

    log = logging.getLogger(__name__ + ".ShowClusterTemplate")

    def get_parser(self, prog_name):
        parser = super(ShowClusterTemplate, self).get_parser(prog_name)
        parser.add_argument(
            "cluster_template",
            metavar="<cluster-template>",
            help="Name or id of the cluster template to display",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = utils.get_resource(
            client.cluster_templates, parsed_args.cluster_template).to_dict()

        _format_ct_output(self.app, data)
        data = utils.prepare_data(data, CT_FIELDS)

        return self.dict2columns(data)


class DeleteClusterTemplate(command.Command):
    """Deletes cluster template"""

    log = logging.getLogger(__name__ + ".DeleteClusterTemplate")

    def get_parser(self, prog_name):
        parser = super(DeleteClusterTemplate, self).get_parser(prog_name)
        parser.add_argument(
            "cluster_template",
            metavar="<cluster-template>",
            nargs="+",
            help="Name(s) or id(s) of the cluster template(s) to delete",
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        for ct in parsed_args.cluster_template:
            ct_id = utils.get_resource_id(client.cluster_templates, ct)
            client.cluster_templates.delete(ct_id)
            sys.stdout.write(
                'Cluster template "{ct}" has been removed '
                'successfully.\n'.format(ct=ct))


class UpdateClusterTemplate(command.ShowOne):
    """Updates cluster template"""

    log = logging.getLogger(__name__ + ".UpdateClusterTemplate")

    def get_parser(self, prog_name):
        parser = super(UpdateClusterTemplate, self).get_parser(prog_name)

        parser.add_argument(
            'cluster_template',
            metavar="<cluster-template>",
            help="Name or ID of the cluster template [REQUIRED]",
        )
        parser.add_argument(
            '--name',
            metavar="<name>",
            help="New name of the cluster template",
        )
        parser.add_argument(
            '--node-groups',
            metavar="<node-group:instances_count>",
            nargs="+",
            help="List of the node groups(names or IDs) and numbers of"
                 "instances for each one of them"
        )
        parser.add_argument(
            '--anti-affinity',
            metavar="<anti-affinity>",
            nargs="+",
            help="List of processes that should be added to an anti-affinity "
                 "group"
        )
        parser.add_argument(
            '--description',
            metavar="<description>",
            help='Description of the cluster template'
        )
        autoconfig = parser.add_mutually_exclusive_group()
        autoconfig.add_argument(
            '--autoconfig-enable',
            action='store_true',
            help='Instances of the cluster will be '
                 'automatically configured',
            dest='use_autoconfig'
        )
        autoconfig.add_argument(
            '--autoconfig-disable',
            action='store_false',
            help='Instances of the cluster will not be '
                 'automatically configured',
            dest='use_autoconfig'
        )
        public = parser.add_mutually_exclusive_group()
        public.add_argument(
            '--public',
            action='store_true',
            help='Make the cluster template public '
                 '(Visible from other projects)',
            dest='is_public'
        )
        public.add_argument(
            '--private',
            action='store_false',
            help='Make the cluster template private '
                 '(Visible only from this tenant)',
            dest='is_public'
        )
        protected = parser.add_mutually_exclusive_group()
        protected.add_argument(
            '--protected',
            action='store_true',
            help='Make the cluster template protected',
            dest='is_protected'
        )
        protected.add_argument(
            '--unprotected',
            action='store_false',
            help='Make the cluster template unprotected',
            dest='is_protected'
        )
        parser.add_argument(
            '--json',
            metavar='<filename>',
            help='JSON representation of the cluster template. Other '
                 'arguments will not be taken into account if this one is '
                 'provided'
        )
        parser.add_argument(
            '--shares',
            metavar='<filename>',
            help='JSON representation of the manila shares'
        )
        parser.add_argument(
            '--configs',
            metavar='<filename>',
            help='JSON representation of the cluster template configs'
        )
        parser.add_argument(
            '--domain-name',
            metavar='<domain-name>',
            default=None,
            help='Domain name for instances of this cluster template. This '
                 'option is available if \'use_designate\' config is True'
        )
        parser.set_defaults(is_public=None, is_protected=None,
                            use_autoconfig=None)
        return parser

    def _take_action(self, client, parsed_args, ct_id):
        if parsed_args.json:
            blob = osc_utils.read_blob_file_contents(parsed_args.json)
            try:
                template = json.loads(blob)
            except ValueError as e:
                raise exceptions.CommandError(
                    'An error occurred when reading '
                    'template from file %s: %s' % (parsed_args.json, e))
            data = client.cluster_templates.update(
                ct_id, **template).to_dict()
        else:
            plugin, plugin_version, node_groups = None, None, None
            if parsed_args.node_groups:
                plugin, plugin_version, node_groups = (
                    utils._cluster_templates_configure_ng(
                        self.app, parsed_args.node_groups, client))

            configs = None
            if parsed_args.configs:
                blob = osc_utils.read_blob_file_contents(parsed_args.configs)
                try:
                    configs = json.loads(blob)
                except ValueError as e:
                    raise exceptions.CommandError(
                        'An error occurred when reading '
                        'configs from file %s: %s' % (parsed_args.configs, e))

            shares = None
            if parsed_args.shares:
                blob = osc_utils.read_blob_file_contents(parsed_args.shares)
                try:
                    shares = json.loads(blob)
                except ValueError as e:
                    raise exceptions.CommandError(
                        'An error occurred when reading '
                        'shares from file %s: %s' % (parsed_args.shares, e))

            data = utils.update_cluster_template(self.app, client, plugin,
                                                 plugin_version, parsed_args,
                                                 configs, shares, node_groups,
                                                 ct_id)

        return data

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        ct_id = utils.get_resource_id(
            client.cluster_templates, parsed_args.cluster_template)

        data = self._take_action(client, parsed_args, ct_id)

        _format_ct_output(self.app, data)
        data = utils.prepare_data(data, CT_FIELDS)

        return self.dict2columns(data)


class ImportClusterTemplate(command.ShowOne):
    """Imports cluster template"""

    log = logging.getLogger(__name__ + ".ImportClusterTemplate")

    def get_parser(self, prog_name):
        parser = super(ImportClusterTemplate, self).get_parser(prog_name)

        parser.add_argument(
            'json',
            metavar="<json>",
            help="JSON containing cluster template",
        )
        parser.add_argument(
            '--name',
            metavar="<name>",
            help="Name of the cluster template",
        )
        parser.add_argument(
            '--default-image-id',
            metavar="<default_image_id>",
            help="Default image ID to be used",
        )
        parser.add_argument(
            '--node-groups',
            metavar="<node-group:instances_count>",
            nargs="+",
            required=True,
            help="List of the node groups(names or IDs) and numbers of "
                 "instances for each one of them"
        )
        return parser

    def _take_action(self, client, parsed_args):
        if (not parsed_args.node_groups):
            raise exceptions.CommandError('--node_groups should be specified')

        blob = osc_utils.read_blob_file_contents(parsed_args.json)
        try:
            template = json.loads(blob)
        except ValueError as e:
            raise exceptions.CommandError(
                'An error occurred when reading '
                'template from file %s: %s' % (parsed_args.json, e))

        if parsed_args.default_image_id:
            template['cluster_template']['default_image_id'] = (
                parsed_args.default_image_id)
        else:
            template['cluster_template']['default_image_id'] = None

        if parsed_args.name:
            template['cluster_template']['name'] = parsed_args.name

        if 'neutron_management_network' in template['cluster_template']:
            template['cluster_template']['net_id'] = (
                template['cluster_template'].pop('neutron_management_network'))

        plugin, plugin_version, node_groups = (
            utils._cluster_templates_configure_ng_configure_node_groups(
                self.app, parsed_args.node_groups, client))
        if (('plugin_version' in template['cluster_template'] and
                template['cluster_template']['plugin_version'] !=
                plugin_version) or
                ('plugin' in template['cluster_template'] and
                    template['cluster_template']['plugin'] != plugin)):
            raise exceptions.CommandError(
                'Plugin of plugin version do not match between template '
                'and given node group templates')
        template['cluster_template']['node_groups'] = node_groups

        data = client.cluster_templates.create(
            **template['cluster_template']).to_dict()

        return data

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing

        data = self._take_action(client, parsed_args)

        _format_ct_output(self.app, data)
        data = utils.prepare_data(data, CT_FIELDS)

        return self.dict2columns(data)


class ExportClusterTemplate(command.Command):
    """Export cluster template to JSON"""

    log = logging.getLogger(__name__ + ".ExportClusterTemplate")

    def get_parser(self, prog_name):
        parser = super(ExportClusterTemplate, self).get_parser(prog_name)
        parser.add_argument(
            "cluster_template",
            metavar="<cluster-template>",
            help="Name or id of the cluster template to export",
        )
        parser.add_argument(
            "--file",
            metavar="<filename>",
            help="Name of the file cluster template should be exported to "
                 "If not provided, print to stdout"
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        client = self.app.client_manager.data_processing
        ngt_id = utils.get_resource_id(
            client.cluster_templates, parsed_args.cluster_template)
        response = client.cluster_templates.export(ngt_id)
        result = json.dumps(response._info, indent=4)+"\n"
        if parsed_args.file:
            with open(parsed_args.file, "w+") as file:
                file.write(result)
        else:
            sys.stdout.write(result)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from osc_lib.tests import utils as osc_u
import testtools
from unittest import mock

from saharaclient.api import job_binaries as api_jb
from saharaclient.osc.v1 import job_binaries as osc_jb
from saharaclient.tests.unit.osc.v1 import fakes


JOB_BINARY_INFO = {
    "name": 'job-binary',
    "description": 'descr',
    "id": 'jb_id',
    "is_protected": False,
    "is_public": False,
    "url": 'swift://cont/test'
}


class TestJobBinaries(fakes.TestDataProcessing):
    def setUp(self):
        super(TestJobBinaries, self).setUp()
        self.jb_mock = self.app.client_manager.data_processing.job_binaries
        self.jb_mock.reset_mock()
        self.app.api_version['data_processing'] = '1'


class TestCreateJobBinary(TestJobBinaries):
    # TODO(apavlov): check for creation with --json
    def setUp(self):
        super(TestCreateJobBinary, self).setUp()
        self.jb_mock.create.return_value = api_jb.JobBinaries(
            None, JOB_BINARY_INFO)
        self.jbi_mock = (self.app.client_manager.
                         data_processing.job_binary_internals)
        self.jbi_mock.create.return_value = mock.Mock(id='jbi_id')
        self.jbi_mock.reset_mock()

        # Command to test
        self.cmd = osc_jb.CreateJobBinary(self.app, None)

    def test_job_binary_create_swift_public_protected(self):
        arglist = ['--name', 'job-binary', '--url', 'swift://cont/test',
                   '--username', 'user', '--password', 'pass', '--public',
                   '--protected']
        verifylist = [('name', 'job-binary'), ('url', 'swift://cont/test'),
                      ('username', 'user'), ('password', 'pass'),
                      ('public', True), ('protected', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.jb_mock.create.assert_called_once_with(
            description=None, extra={'password': 'pass', 'user': 'user'},
            is_protected=True, is_public=True, name='job-binary',
            url='swift://cont/test')

        # Check that columns are correct
        expected_columns = ('Description', 'Id', 'Is protected', 'Is public',
                            'Name', 'Url')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('descr', 'jb_id', False, False, 'job-binary',
                         'swift://cont/test')
        self.assertEqual(expected_data, data)

    def test_job_binary_create_internal(self):
        m_open = mock.mock_open()
        with mock.patch('six.moves.builtins.open', m_open, create=True):
            arglist = ['--name', 'job-binary', '--data', 'filepath']
            verifylist = [('name', 'job-binary'), ('data', 'filepath')]

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            self.cmd.take_action(parsed_args)

            # Check that correct arguments were passed
            self.jb_mock.create.assert_called_once_with(
                description=None, extra=None, is_protected=False,
                is_public=False, name='job-binary', url='internal-db://jbi_id')

            self.jbi_mock.create.assert_called_once_with('job-binary', '')

    def test_job_binary_create_mutual_exclusion(self):
        arglist = ['job-binary', '--name', 'job-binary', '--access-key', 'ak',
                   '--secret-key', 'sk', '--url', 's3://abc/def',
                   '--password', 'pw']

        with testtools.ExpectedException(osc_u.ParserException):
            self.check_parser(self.cmd, arglist, mock.Mock())


class TestListJobBinaries(TestJobBinaries):
    def setUp(self):
        super(TestListJobBinaries, self).setUp()
        self.jb_mock.list.return_value = [api_jb.JobBinaries(
            None, JOB_BINARY_INFO)]

        # Command to test
        self.cmd = osc_jb.ListJobBinaries(self.app, None)

    def test_job_binary_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Url']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('job-binary', 'jb_id', 'swift://cont/test')]
        self.assertEqual(expected_data, list(data))

    def test_job_binary_list_long(self):
        arglist = ['--long']
        verifylist = [('long', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Url', 'Description', 'Is public',
                            'Is protected']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('job-binary', 'jb_id', 'swift://cont/test', 'descr',
                          False, False)]
        self.assertEqual(expected_data, list(data))

    def test_job_binary_list_extra_search_opts(self):
        arglist = ['--name', 'bin']
        verifylist = [('name', 'bin')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Name', 'Id', 'Url']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('job-binary', 'jb_id', 'swift://cont/test')]
        self.assertEqual(expected_data, list(data))


class TestShowJobBinary(TestJobBinaries):
    def setUp(self):
        super(TestShowJobBinary, self).setUp()
        self.jb_mock.find_unique.return_value = api_jb.JobBinaries(
            None, JOB_BINARY_INFO)

        # Command to test
        self.cmd = osc_jb.ShowJobBinary(self.app, None)

    def test_job_binary_show(self):
        arglist = ['job-binary']
        verifylist = [('job_binary', 'job-binary')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.jb_mock.find_unique.assert_called_once_with(name='job-binary')

        # Check that columns are correct
        expected_columns = ('Description', 'Id', 'Is protected', 'Is public',
                            'Name', 'Url')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('descr', 'jb_id', False, False, 'job-binary',
                         'swift://cont/test')
        self.assertEqual(expected_data, data)


class TestDeleteJobBinary(TestJobBinaries):
    def setUp(self):
        super(TestDeleteJobBinary, self).setUp()
        self.jb_mock.find_unique.return_value = api_jb.JobBinaries(
            None, JOB_BINARY_INFO)

        # Command to test
        self.cmd = osc_jb.DeleteJobBinary(self.app, None)

    def test_job_binary_delete(self):
        arglist = ['job-binary']
        verifylist = [('job_binary', ['job-binary'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.jb_mock.delete.assert_called_once_with('jb_id')


class TestUpdateJobBinary(TestJobBinaries):
    def setUp(self):
        super(TestUpdateJobBinary, self).setUp()
        self.jb_mock.find_unique.return_value = api_jb.JobBinaries(
            None, JOB_BINARY_INFO)
        self.jb_mock.update.return_value = api_jb.JobBinaries(
            None, JOB_BINARY_INFO)

        # Command to test
        self.cmd = osc_jb.UpdateJobBinary(self.app, None)

    def test_job_binary_update_all_options(self):
        arglist = ['job-binary', '--name', 'job-binary', '--description',
                   'descr', '--username', 'user', '--password', 'pass',
                   '--public', '--protected']

        verifylist = [('job_binary', 'job-binary'), ('name', 'job-binary'),
                      ('description', 'descr'), ('username', 'user'),
                      ('password', 'pass'), ('is_public', True),
                      ('is_protected', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.jb_mock.update.assert_called_once_with(
            'jb_id',
            {'is_public': True, 'description': 'descr', 'is_protected': True,
             'name': 'job-binary',
             'extra': {'password': 'pass', 'user': 'user'}})

        # Check that columns are correct
        expected_columns = ('Description', 'Id', 'Is protected', 'Is public',
                            'Name', 'Url')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('descr', 'jb_id', False, False, 'job-binary',
                         'swift://cont/test')
        self.assertEqual(expected_data, data)

    def test_job_binary_update_private_unprotected(self):
        arglist = ['job-binary', '--private', '--unprotected']

        verifylist = [('job_binary', 'job-binary'), ('is_public', False),
                      ('is_protected', False)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.jb_mock.update.assert_called_once_with(
            'jb_id', {'is_public': False, 'is_protected': False})

    def test_job_binary_update_nothing_updated(self):
        arglist = ['job-binary']

        verifylist = [('job_binary', 'job-binary')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.jb_mock.update.assert_called_once_with(
            'jb_id', {})

    def test_job_binary_update_mutual_exclusion(self):
        arglist = ['job-binary', '--name', 'job-binary', '--access-key', 'ak',
                   '--secret-key', 'sk', '--url', 's3://abc/def',
                   '--password', 'pw']

        with testtools.ExpectedException(osc_u.ParserException):
            self.check_parser(self.cmd, arglist, mock.Mock())


class TestDownloadJobBinary(TestJobBinaries):
    def setUp(self):
        super(TestDownloadJobBinary, self).setUp()
        self.jb_mock.get_file.return_value = 'data'
        self.jb_mock.find_unique.return_value = api_jb.JobBinaries(
            None, JOB_BINARY_INFO)

        # Command to test
        self.cmd = osc_jb.DownloadJobBinary(self.app, None)

    def test_download_job_binary_default_file(self):
        m_open = mock.mock_open()
        with mock.patch('six.moves.builtins.open', m_open, create=True):
            arglist = ['job-binary']
            verifylist = [('job_binary', 'job-binary')]

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            self.cmd.take_action(parsed_args)

            # Check that correct arguments was passed
            self.jb_mock.get_file.assert_called_once_with(
                'jb_id')

            # Check that data will be saved to the right file
            self.assertEqual('job-binary', m_open.call_args[0][0])

    def test_download_job_binary_specified_file(self):
        m_open = mock.mock_open()
        with mock.patch('six.moves.builtins.open', m_open, create=True):
            arglist = ['job-binary', '--file', 'test']
            verifylist = [('job_binary', 'job-binary'), ('file', 'test')]

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            self.cmd.take_action(parsed_args)

            # Check that correct arguments was passed
            self.jb_mock.get_file.assert_called_once_with(
                'jb_id')

            # Check that data will be saved to the right file
            self.assertEqual('test', m_open.call_args[0][0])



# Copyright (c) 2015 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from saharaclient.api import base


class JobType(base.Resource):
    resource_name = 'JobType'


class JobTypesManager(base.ResourceManager):
    resource_class = JobType

    def list(self, search_opts=None):
        """Get a list of job types supported by plugins."""
        query = base.get_query_string(search_opts)
        return self._list('/job-types%s' % query, 'job_types')



# Copyright (c) 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from osc_lib.tests import utils as osc_utils
from unittest import mock

from saharaclient.api.v2 import jobs as api_j
from saharaclient.osc.v2 import jobs as osc_j
from saharaclient.tests.unit.osc.v1 import test_jobs as tj_v1

JOB_INFO = {
    "is_public": False,
    "id": "j_id",
    "interface": [],
    "is_protected": False,
    "input_id": 'input_id',
    "output_id": 'output_id',
    "job_template_id": "job_template_id",
    "cluster_id": 'cluster_id',
    "start_time": "start",
    "end_time": "end",
    "engine_job_id": "engine_job_id",
    "info": {
        "status": 'SUCCEEDED'
    },
    "job_configs": {
        "configs": {
            "config1": "1",
            "config2": "2"
        },
        "args": [
            "arg1",
            "arg2"
        ],
        "params": {
            "param2": "value2",
            "param1": "value1"
        }
    }
}


class TestJobs(tj_v1.TestJobs):
    def setUp(self):
        super(TestJobs, self).setUp()
        self.app.api_version['data_processing'] = '2'
        self.j_mock = self.app.client_manager.data_processing.jobs
        self.j_mock.reset_mock()


class TestExecuteJob(TestJobs):
    # TODO(apavlov): check for execution with --interface, --configs, --json
    def setUp(self):
        super(TestExecuteJob, self).setUp()
        self.j_mock.create.return_value = api_j.Job(None, JOB_INFO)
        self.ds_mock = self.app.client_manager.data_processing.data_sources
        self.ds_mock.find_unique.return_value = mock.Mock(id='ds_id')
        self.c_mock = self.app.client_manager.data_processing.clusters
        self.c_mock.find_unique.return_value = mock.Mock(id='cluster_id')
        self.jt_mock = self.app.client_manager.data_processing.job_templates
        self.jt_mock.find_unique.return_value = mock.Mock(id='job_template_id')
        self.ds_mock.reset_mock()
        self.c_mock.reset_mock()
        self.jt_mock.reset_mock()

        # Command to test
        self.cmd = osc_j.ExecuteJob(self.app, None)

    def test_job_execute_minimum_options(self):
        arglist = ['--job-template', 'job-template', '--cluster', 'cluster']
        verifylist = [('job_template', 'job-template'), ('cluster', 'cluster')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.j_mock.create.assert_called_once_with(
            cluster_id='cluster_id', configs={}, input_id=None,
            interface=None, is_protected=False, is_public=False,
            job_template_id='job_template_id', output_id=None)

    def test_job_execute_with_input_output_option(self):
        arglist = ['--job-template', 'job-template', '--cluster', 'cluster',
                   '--input', 'input', '--output', 'output']
        verifylist = [('job_template', 'job-template'), ('cluster', 'cluster'),
                      ('input', 'input'), ('output', 'output')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        self.j_mock.create.assert_called_once_with(
            cluster_id='cluster_id', configs={}, input_id='ds_id',
            interface=None, is_protected=False, is_public=False,
            job_template_id='job_template_id', output_id='ds_id')

        # without option --output
        arglist = ['--job-template', 'job-template', '--cluster', 'cluster',
                   '--input', 'input']
        verifylist = [('job_template', 'job-template'), ('cluster', 'cluster'),
                      ('input', 'input')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        self.j_mock.create.assert_called_with(
            cluster_id='cluster_id', configs={}, input_id='ds_id',
            interface=None, is_protected=False, is_public=False,
            job_template_id='job_template_id', output_id=None)

        # without options --output and --input
        arglist = ['--job-template', 'job-template', '--cluster', 'cluster']
        verifylist = [('job_template', 'job-template'), ('cluster', 'cluster')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        self.j_mock.create.assert_called_with(
            cluster_id='cluster_id', configs={}, input_id=None,
            interface=None, is_protected=False, is_public=False,
            job_template_id='job_template_id', output_id=None)

    def test_job_execute_all_options(self):
        arglist = ['--job-template', 'job-template', '--cluster', 'cluster',
                   '--input', 'input', '--output', 'output', '--params',
                   'param1:value1', 'param2:value2', '--args', 'arg1', 'arg2',
                   '--configs', 'config1:1', 'config2:2', '--public',
                   '--protected']

        verifylist = [('job_template', 'job-template'), ('cluster', 'cluster'),
                      ('input', 'input'), ('output', 'output'),
                      ('params', ['param1:value1', 'param2:value2']),
                      ('args', ['arg1', 'arg2']),
                      ('configs', ['config1:1', 'config2:2']),
                      ('public', True),
                      ('protected', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.j_mock.create.assert_called_once_with(
            cluster_id='cluster_id',
            configs={'configs': {'config1': '1', 'config2': '2'},
                     'args': ['arg1', 'arg2'],
                     'params': {'param2': 'value2', 'param1': 'value1'}},
            input_id='ds_id', interface=None, is_protected=True,
            is_public=True, job_template_id='job_template_id',
            output_id='ds_id')

        # Check that columns are correct
        expected_columns = ('Cluster id', 'End time', 'Engine job id', 'Id',
                            'Input id', 'Is protected', 'Is public',
                            'Job template id', 'Output id', 'Start time',
                            'Status')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('cluster_id', 'end', 'engine_job_id', 'j_id',
                         'input_id', False, False, 'job_template_id',
                         'output_id', 'start', 'SUCCEEDED')
        self.assertEqual(expected_data, data)


class TestListJobs(TestJobs):
    def setUp(self):
        super(TestListJobs, self).setUp()
        self.j_mock.list.return_value = [api_j.Job(None, JOB_INFO)]

        # Command to test
        self.cmd = osc_j.ListJobs(self.app, None)

    def test_jobs_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Id', 'Cluster id', 'Job template id', 'Status']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('j_id', 'cluster_id', 'job_template_id',
                          'SUCCEEDED')]
        self.assertEqual(expected_data, list(data))

    def test_jobs_list_long(self):
        arglist = ['--long']
        verifylist = [('long', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Id', 'Cluster id', 'Job template id', 'Status',
                            'Start time', 'End time']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('j_id', 'cluster_id', 'job_template_id', 'SUCCEEDED',
                          'start', 'end')]
        self.assertEqual(expected_data, list(data))

    def test_jobs_list_extra_search_opts(self):
        arglist = ['--status', 'succeeded']
        verifylist = [('status', 'succeeded')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Id', 'Cluster id', 'Job template id', 'Status']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [('j_id', 'cluster_id', 'job_template_id',
                          'SUCCEEDED')]
        self.assertEqual(expected_data, list(data))


class TestShowJob(TestJobs):
    def setUp(self):
        super(TestShowJob, self).setUp()
        self.j_mock.get.return_value = api_j.Job(None, JOB_INFO)

        # Command to test
        self.cmd = osc_j.ShowJob(self.app, None)

    def test_job_show(self):
        arglist = ['job_id']
        verifylist = [('job', 'job_id')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.j_mock.get.assert_called_once_with('job_id')

        # Check that columns are correct
        expected_columns = ('Cluster id', 'End time', 'Engine job id', 'Id',
                            'Input id', 'Is protected', 'Is public',
                            'Job template id', 'Output id', 'Start time',
                            'Status')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('cluster_id', 'end', 'engine_job_id', 'j_id',
                         'input_id', False, False, 'job_template_id',
                         'output_id', 'start', 'SUCCEEDED')
        self.assertEqual(expected_data, data)


class TestDeleteJob(TestJobs):
    def setUp(self):
        super(TestDeleteJob, self).setUp()
        self.j_mock.get.return_value = api_j.Job(None, JOB_INFO)

        # Command to test
        self.cmd = osc_j.DeleteJob(self.app, None)

    def test_job_delete(self):
        arglist = ['job_id']
        verifylist = [('job', ['job_id'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.j_mock.delete.assert_called_once_with('job_id')


class TestUpdateJob(TestJobs):
    def setUp(self):
        super(TestUpdateJob, self).setUp()
        self.j_mock.get.return_value = api_j.Job(None, JOB_INFO)
        self.j_mock.update.return_value = mock.Mock(job=JOB_INFO.copy())

        # Command to test
        self.cmd = osc_j.UpdateJob(self.app, None)

    def test_job_update_no_options(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_job_update_nothing_updated(self):
        arglist = ['job_id']

        verifylist = [('job', 'job_id')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.j_mock.update.assert_called_once_with('job_id')

    def test_job_update_public_protected(self):
        arglist = ['job_id', '--public', '--protected']

        verifylist = [('job', 'job_id'), ('is_public', True),
                      ('is_protected', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.j_mock.update.assert_called_once_with(
            'job_id', is_protected=True, is_public=True)

        # Check that columns are correct
        expected_columns = ('Cluster id', 'End time', 'Engine job id', 'Id',
                            'Input id', 'Is protected', 'Is public',
                            'Job template id', 'Output id', 'Start time',
                            'Status')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ('cluster_id', 'end', 'engine_job_id', 'j_id',
                         'input_id', False, False, 'job_template_id',
                         'output_id', 'start', 'SUCCEEDED')
        self.assertEqual(expected_data, data)

    def test_job_update_private_unprotected(self):
        arglist = ['job_id', '--private', '--unprotected']

        verifylist = [('job', 'job_id'), ('is_public', False),
                      ('is_protected', False)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.j_mock.update.assert_called_once_with(
            'job_id', is_protected=False, is_public=False)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from saharaclient.api import base
from saharaclient.tests.unit import base as test_base


class ManagerTest(test_base.BaseTestCase):

    def setUp(self):
        super(ManagerTest, self).setUp()
        self.man = test_base.TestManager(self.client)

    def test_find(self):
        self.man.list = mock.MagicMock(
            return_value=[mock.Mock(test='foo'),
                          mock.Mock(test='bar')]
        )

        self.assertEqual(2, len(self.man.find()))
        self.assertEqual(1, len(self.man.find(test='foo')))
        self.assertEqual(0, len(self.man.find(test='baz')))

    def test_find_unique(self):
        expected = mock.Mock(test='foo')
        self.man.list = mock.MagicMock(
            return_value=[expected,
                          mock.Mock(test='bar')]
        )

        ex = self.assertRaises(base.APIException,
                               self.man.find_unique, test='baz')
        self.assertEqual(404, ex.error_code)
        ex = self.assertRaises(base.APIException, self.man.find_unique)
        self.assertEqual(409, ex.error_code)
        self.assertEqual(expected, self.man.find_unique(test='foo'))
